--	GameState.lua - Main script run by LuaGameState.cpp.
--  ----------------------------------------------------------------------------
--	This file is part of 'EXCOSC Base' a simple OSC-aware game skeleton.
--	Copyright (C) 2013  Niall Moody
--	
--	This program is free software: you can redistribute it and/or modify
--	it under the terms of the GNU General Public License as published by
--	the Free Software Foundation, either version 3 of the License, or
--	(at your option) any later version.
--
--	This program is distributed in the hope that it will be useful,
--	but WITHOUT ANY WARRANTY; without even the implied warranty of
--	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--	GNU General Public License for more details.
--
--	You should have received a copy of the GNU General Public License
--	along with this program.  If not, see <http://www.gnu.org/licenses/>.
--	----------------------------------------------------------------------------

--	True if we're on the title screen.
title = true

--	Whether the players are dead.
playersDead = {false, false}
--	Whether the players' shields are up.
playersShields = {0.0, 0.0}
--	Whether to drop the players' shields.
playersKeys = {false, false}
--	Ditto for gamepads.
playersJoys = {0.0, 0.0}
--	Keeps track of how many bullets each player has.
playersNumBullets = {1, 1}
--	Keeps track of the bullets fired by the players.
playersBullets = {TwoFloats(322.0, 422.0), TwoFloats(952.0, 422.0)}
--	The position of the players' shields.
playersShieldsPos = {}
--	Flings the players' shields away after 10 seconds.
playersShieldsFling = {TwoFloats(-0.7, -0.7), TwoFloats(0.7, -0.7)}
--	Which player won.
winner = 0

--	Used for the 10s timer.
lastTimer = 0.0
--	The 10s timer.
tenSecondTimer = 10
--	The images for the timer.
timerImages = {"Number1",
			   "Number2",
			   "Number3",
			   "Number4",
			   "Number5",
			   "Number6",
			   "Number7",
			   "Number8",
			   "Number9",
			   "Number10"}
--	Used to scale up the timer images.
timerScale = 0.0

--	Used to make sure we don't quit to title too soon.
titleTimer = 0

--	Used so we can slow things down once both bullets are fired.
bulletSpeed = 32.0
--	Used so we can slow things down once both bullets are fired.
shieldSpeed = 1.0

--	The clangs.
clangPos = {TwoFloats(346.0, 422.0), TwoFloats(932.0, 422.0)}
--	The clangs.
clangSize = {0.0, 0.0}
--	The clangs.
clangs = {false, false}
--	The ows.
owPos = {TwoFloats(346.0, 422.0), TwoFloats(932.0, 422.0)}
--	The ows.
owSize = {0.0, 0.0}
--	The ows.
ows = {false, false}

--	Whether we've already started the music or not.
music = false

--------------------------------------------------------------------------------
--	Called when the state is first initialised.
function initialise(app)
	playersShieldsPos[1] = TwoFloats(128.0+40.0, 236.0)
	playersShieldsPos[2] = TwoFloats(app:getDrawingAreaSize().x-128.0-256.0-40.0, 236.0)

	winner = 0
	tenSecondTimer = 10
	bulletSpeed = 32.0
	shieldSpeed = 1.0
	for i=1,2 do
		playersDead[i] = false
		playersShields[i] = 0.0
		playersKeys[i] = false
		playersNumBullets[i] = 1
	end
	playersBullets[1] = TwoFloats(322.0, 422.0)
	playersBullets[2] = TwoFloats(952.0, 422.0)

	if not music then
		music = true
		app:playMusic("StarlitShootoutMusic")
	end
end

--------------------------------------------------------------------------------
--	Used to update game logic etc.
function update(app)
	if (winner == 0) and
       (playersNumBullets[1] == 0) and
	   (playersNumBullets[2] == 0)then
		if (bulletSpeed > 4.0) then
			bulletSpeed = bulletSpeed - 2.0
		end
		if (shieldSpeed > 0.0625) then
			shieldSpeed = shieldSpeed - 0.075
		end
	else
		if (bulletSpeed < 32.0) then
			bulletSpeed = bulletSpeed + 2.0
		end
		if (shieldSpeed < 1.0) then
			shieldSpeed = shieldSpeed + 0.075
		end
	end

	if not title then
		if (tenSecondTimer > 0) then
			--Update the timer.
			local tempTimer = os.clock() * 5.0

			if (tempTimer-lastTimer) >= 1.0 then
				tenSecondTimer = tenSecondTimer - 1
				lastTimer = tempTimer
				timerScale = 0.0

				if (tenSecondTimer > 0) then
					app:triggerSound("Countdown", 1.0, 0.5)
				elseif (tenSecondTimer == 0) then
					app:triggerSound("CountdownEnd", 1.0, 0.5)
					app:triggerSound("ShieldFling", 1.0, 0.5)
				end

				if (tenSecondTimer == 0) and
				   (playersNumBullets[1] < 1) and
				   (playersNumBullets[2] < 1) then
					playersDead[1] = true
					playersDead[2] = true
					titleTimer = 30
					winner = 3

					app:triggerSound("Ow", 1.0, 0.5)
				end
			end

			if (timerScale < 1.0) then
				timerScale = timerScale + 0.05
			end

			--Handle the player's shields.
			for i=1,2 do
				if playersKeys[i] and (playersShields[i] < 1.0) then
					playersShields[i] = playersShields[i] + (0.1 * shieldSpeed)
				elseif not playersKeys[i] and (playersJoys[i] <= 0.0) and (playersShields[i] > 0.0) then
					playersShields[i] = playersShields[i] - (0.1 * shieldSpeed)
				end
			end
	
			if playersDead[1] then
				playersShieldsPos[1] = playersShieldsPos[1] + (playersShieldsFling[1] * 12.0 * shieldSpeed)
				playersShields[1] = playersShields[1] + (0.2 * shieldSpeed)
			end
			if playersDead[2] then
				playersShieldsPos[2] = playersShieldsPos[2] + (playersShieldsFling[2] * 12.0 * shieldSpeed)
				playersShields[2] = playersShields[2] + (0.2 * shieldSpeed)
			end
		else
			--Otherwise fling them into the sky.
			if (winner == 2) or (winner == 0) or (winner == 3) then
				playersShieldsPos[1] = playersShieldsPos[1] + (playersShieldsFling[1] * 12.0 * shieldSpeed)
				playersShields[1] = playersShields[1] + (0.2 * shieldSpeed)
			end
			
			if (winner == 1) or (winner == 0) or (winner == 3) then
				playersShieldsPos[2] = playersShieldsPos[2] + (playersShieldsFling[2] * 12.0 * shieldSpeed)
				playersShields[2] = playersShields[2] + (0.2 * shieldSpeed)
			end
		end

		--Handle the bullets.
		if (playersNumBullets[1] == 0) then
			local lastPos = TwoFloats(playersBullets[1])
		
			playersBullets[1].x = playersBullets[1].x + bulletSpeed

			if (winner == 0) then
				--Test against player 1's shield.
				if (playersShields[1] < 0.66) and
				   (lastPos.x < 346.0) and
				   (playersBullets[1].x >= 346.0) then
				   playersNumBullets[1] = -1
				   clangs[1] = true
				   clangSize[1] = 0.02
				   app:triggerSound("Clang", 1.0, 0.0)
				--Test against player 2's shield.
				elseif (playersShields[2] < 0.66) and
					   (lastPos.x < 932.0) and
					   (playersBullets[1].x >= 932.0) then
				   playersNumBullets[1] = -1
				   clangs[2] = true
				   clangSize[2] = 0.02
				   app:triggerSound("Clang", 1.0, 1.0)
				--Test against player 2.
				elseif (playersBullets[1].x >= 1006.0) then
					playersDead[2] = true
					tenSecondTimer = 0
					winner = 1
					titleTimer = 30
					ows[2] = true
					owSize[2] = 0.02
					app:triggerSound("Ow", 1.0, 1.0)
					app:triggerSound("ShieldFling", 1.0, 1.0)
				end
			end
		end

		if (playersNumBullets[2] == 0) then
			local lastPos = TwoFloats(playersBullets[2])
		
			playersBullets[2].x = playersBullets[2].x - bulletSpeed

			if (winner == 0) then
				--Test against player 2's shield.
				if (playersShields[2] < 0.66) and
				   (lastPos.x > 932.0) and
				   (playersBullets[2].x <= 932.0) then
					playersNumBullets[2] = -1
				   clangs[2] = true
				   clangSize[2] = 0.02
				   app:triggerSound("Clang", 1.0, 1.0)
				--Test against player 1's shield.
				elseif (playersShields[1] < 0.66) and
					   (lastPos.x > 346.0) and
					   (playersBullets[2].x <= 346.0) then
					playersNumBullets[2] = -1
				   clangs[1] = true
				   clangSize[1] = 0.02
				   app:triggerSound("Clang", 1.0, 0.0)
				--Test against player 1.
				elseif (playersBullets[2].x <= 275.0) then
					playersDead[1] = true
					tenSecondTimer = 0
					winner = 2
					titleTimer = 30
					ows[1] = true
					owSize[1] = 0.02
					app:triggerSound("Ow", 1.0, 0.0)
					app:triggerSound("ShieldFling", 1.0, 0.0)
				end
			end
		end
	end

	if (titleTimer > 0) then
		titleTimer = titleTimer - 1
	end

	for i=1,2 do
		if (clangSize[i] > 0.0) then
			clangSize[i] = clangSize[i] + 0.04
			if (clangSize[i] > 2.0) then
				clangSize[i] = 0.0
			elseif (clangSize[i] > 1.0) then
				clangSize[i] = clangSize[i] + 0.04
			end
		end
		if (owSize[i] > 0.0) then
			owSize[i] = owSize[i] + 0.04
			if (owSize[i] > 2.0) then
				owSize[i] = 0.0
			elseif (owSize[i] > 1.0) then
				owSize[i] = owSize[i] + 0.04
			end
		end
	end
end

--------------------------------------------------------------------------------
--	Draws the game.
function draw(d)
	d:fillRect(TwoFloats(0.0), d:getDrawingAreaSize(), ThreeFloats(1.0), 1.0)

	d:drawImage("Background",
				TwoFloats(0.0),
				TwoFloats(1.0),
				ThreeFloats(1.0), 1.0)
	
	if title then
		titleOffset = TwoFloats(512.0, 256.0)

		d:drawImage("Title",
					(d:getDrawingAreaSize()*0.5)-titleOffset,
					TwoFloats(1.0),
					ThreeFloats(1.0),
					1.0)
	else
		if playersDead[1] then
			d:drawImage("PlayerDead",
						TwoFloats(32.0+384.0, 376.0),
						TwoFloats(-1.0, 1.0),
						ThreeFloats(1.0),
						1.0)
		else
			d:drawImage("Player",
						TwoFloats(128.0+256.0, 236.0),
						TwoFloats(-1.0, 1.0),
						ThreeFloats(1.0),
						1.0)
		end
		d:drawRotImage("PlayerShieldRed",
					   playersShieldsPos[1],
					   playersShields[1] * (math.pi*0.5),
					   TwoFloats(44.0, 164.0),
					   TwoFloats(1.0),
					   ThreeFloats(1.0),
					   1.0)

		if playersDead[2] then
			d:drawImage("PlayerDead",
						TwoFloats(d:getDrawingAreaSize().x-32.0-384.0, 376.0),
						TwoFloats(1.0),
						ThreeFloats(1.0),
						1.0)
		else
			d:drawImage("Player",
						TwoFloats(d:getDrawingAreaSize().x-128.0-256.0, 236.0),
						TwoFloats(1.0),
						ThreeFloats(1.0),
						1.0)
		end
		d:drawRotImage("PlayerShield",
					   playersShieldsPos[2],
					   -(playersShields[2] * (math.pi*0.5)),
					   TwoFloats(205.0, 164.0),
					   TwoFloats(1.0),
					   ThreeFloats(1.0),
					   1.0)

		--Draw bullets.
		if (playersNumBullets[1] == 0) then
			local bulletPos = TwoFloats(playersBullets[1])

			bulletPos.x = bulletPos.x - 11.0 + 32.0
			bulletPos.y = bulletPos.y - 8.0
			d:drawImage("Bullet",
						bulletPos,
						TwoFloats(-1.0, 1.0),
						ThreeFloats(1.0),
						1.0)
		end
		if (playersNumBullets[2] == 0) then
			local bulletPos = TwoFloats(playersBullets[2])

			bulletPos.x = bulletPos.x - 11.0
			bulletPos.y = bulletPos.y - 8.0
			d:drawImage("Bullet",
						bulletPos,
						TwoFloats(1.0),
						ThreeFloats(1.0),
						1.0)
		end

		--Draw SFX.
		for i=1,2 do
			local tempSize = 0.0

			if (clangSize[i] > 0.0) then
				if (clangSize[i] > 1.0) then
					tempSize = 2.0 - clangSize[i]
				else
					tempSize = clangSize[i]
				end
				d:drawImage("Clang",
							clangPos[i]-(128.0 * tempSize),
							TwoFloats(tempSize),
							ThreeFloats(1.0),
							1.0)
			end
			if (owSize[i] > 0.0) then
				if (owSize[i] > 1.0) then
					tempSize = 2.0 - owSize[i]
				else
					tempSize = owSize[i]
				end
				d:drawImage("Ow",
							owPos[i]-(128.0 * tempSize),
							TwoFloats(tempSize),
							ThreeFloats(1.0),
							1.0)
			end
		end

		--Draw timer.
		if (tenSecondTimer > 0) then
			local imagePos = TwoFloats(d:getDrawingAreaSize()*0.5)

			imagePos.y = 128.0
			d:drawImage(timerImages[tenSecondTimer],
						imagePos-(256.0*timerScale*0.5),
						TwoFloats(timerScale*0.5),
						ThreeFloats(1.0),
						1.0)
		end
	end
end

--------------------------------------------------------------------------------
--	Called when the user depresses a key or a joystick button.
function keyDown(app, action, keyPressed, keyModifier)
	if title then
		title = false
		initialise(app)
		lastTimer = os.clock()
	elseif (winner > 0) and (titleTimer <= 0) then
		title = true
	else
		if (keyPressed == "x") and (playersShields[1] < 1.0) then
			playersKeys[1] = true
		elseif ((keyPressed == "z") or (keyPressed == "Joy0Button0")) and
			   not playersDead[1] then
			if (playersNumBullets[1] > 0) then
				playersNumBullets[1] = 0
				app:triggerSound("Shoot", 1.0, 0.0)
			end
		elseif (keyPressed == "CursorDown") and (playersShields[2] < 1.0) then
			playersKeys[2] = true
		elseif ((keyPressed == "CursorLeft") or (keyPressed == "Joy1Button0")) and
			   not playersDead[2] then
			if (playersNumBullets[2] > 0) then
				playersNumBullets[2] = 0
				app:triggerSound("Shoot", 1.0, 1.0)
			end
		end
	end
end

--------------------------------------------------------------------------------
--	Called when the user releases a key or a joystick button.
function keyUp(app, action, keyPressed, keyModifier)
	if (keyPressed == "x") then
		playersKeys[1] = false
	elseif (keyPressed == "z") then
		
	elseif (keyPressed == "CursorDown") then
		playersKeys[2] = false
	elseif (keyPressed == "CursorLeft") then
		
	end
end

--------------------------------------------------------------------------------
--	Called when the user depresses a mouse button.
function mouseDown(app, button, pos)
	
end

--------------------------------------------------------------------------------
--	Called when the user releases a mouse button.
function mouseUp(app, button, pos)

end

--------------------------------------------------------------------------------
--	Called when the user moves the mouse.
function mouseMove(app, pos)
	
end

--------------------------------------------------------------------------------
--	Called when the user moves the mousewheel.
function mouseWheel(app, dir)

end

--------------------------------------------------------------------------------
--	Called when the user moves a joystick.
function joyMove(app, action, value, joystick, axis)
	if (joystick == 0) and (axis == 2) then
		playersShields[1] = value
		playersJoys[1] = value
	elseif (joystick == 1) and (axis == 2) then
		playersShields[2] = value
		playersJoys[2] = value
	end
end

--------------------------------------------------------------------------------
--	Called when we receive an OSC message.
function oscMessageReceived(app, address, value)
	
end
